Package: libr
Type: Package
Title: Libraries, Data Dictionaries, and a Data Step for R
Version: 1.3.4
Author: David J. Bosak
Maintainer: David Bosak <dbosak01@gmail.com>
Description: Contains a set of functions to create data libraries,
    generate data dictionaries, and simulate a data step.
    The libname() function will load a directory of data into 
    a library in one line of code.  The dictionary() function
    will generate data dictionaries for individual
    data frames or an entire library.  And the datestep() function
    will perform row-by-row data processing.
License: CC0
Encoding: UTF-8
URL: https://libr.r-sassy.org, https://github.com/dbosak01/libr
BugReports: https://github.com/dbosak01/libr/issues
Depends: R (>= 3.6.0), common (>= 1.1.0)
Suggests: testthat, knitr, rmarkdown, foreign, magrittr, utils, logr,
        covr, fmtr
Imports: readr, readxl, haven, openxlsx, crayon, dplyr, tibble, tools,
        Rcpp, data.table
RoxygenNote: 7.3.1
VignetteBuilder: knitr
LinkingTo: Rcpp
Built: R 4.4.1; x86_64-apple-darwin24.0.0; 2024-10-20 08:37:43 UTC; unix
